package com.chelvc.framework.oauth.token;

import java.time.Duration;
import java.util.Map;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.oauth.context.OAuthContextHolder;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;

/**
 * 基于Redis令牌管理器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class RedisTokenStore extends JwtTokenStore {
    public RedisTokenStore(@NonNull JwtAccessTokenConverter jwtAccessTokenConverter) {
        super(jwtAccessTokenConverter);
    }

    @Override
    public void storeAccessToken(OAuth2AccessToken token, OAuth2Authentication authentication) {
        Object principal = authentication.getPrincipal();
        if (principal instanceof UserDetails) {
            principal = ((UserDetails) principal).getUsername();
        }
        String key = OAuthContextHolder.key(principal);
        Map<String, ?> values = ImmutableMap.of(
                SessionContextHolder.HEADER_SCOPE, String.valueOf(OAuthContextHolder.getScope(token)),
                String.valueOf(SessionContextHolder.getTerminal()), token.getValue()
        );
        Duration duration = RedisContextHolder.duration(token.getExpiration());
        try {
            RedisContextHolder.put(RedisContextHolder.getDefaultTemplate(), key, values, duration);
        } catch (Exception e) {
            log.warn("Redis token save failed: {}", e.getMessage());
        }
    }
}
